@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
 # pylint: disable-msg=W0613,W0612,W0212,W0511,R0912,C0322,W0704
 # W0511 = XXX (my own todo's)
 
@@ -38,33 +36,40 @@ from yolk.setuptools_support import get_download_uri
 from yolk.plugins import load_plugins
 
 
-class Usage(Exception):
-
-    """Usage exception"""
-
-    def __init__(self, msg):
-        """init"""
-        LOGGER.error("%s" % msg)
-        sys.exit(2)
-
 
 #Functions for obtaining info about packages installed by setuptools
 ##############################################################################
 
 
 def get_pkglist():
-    """Return list of all pkg names"""
+    """
+    Return list of all installed packages
+
+    Note: It returns one project name per pkg no matter how many versions
+    of a particular package is installed
+     
+    @returns: list of project name strings for every installed pkg
+    
+    """
 
     dists = Distributions()
-    pkgs = []
+    projects = []
     for (dist, active) in dists.get_distributions("all"):
-        if dist.project_name not in pkgs:
-            pkgs.append(dist.project_name)
-    return pkgs
+        if dist.project_name not in projects:
+            projects.append(dist.project_name)
+    return projects
 
 
 def show_updates(package_name=""):
-    """Check installed packages for available updates on PyPI"""
+    """
+    Check installed packages for available updates on PyPI
+
+    @param package_name: optional package name to check; checks every
+                         installed pacakge if none specified
+    @type package_name: string
+
+    @returns: None
+    """
     pypi = CheeseShop()
     dists = Distributions()
     if package_name:
@@ -98,7 +103,18 @@ def show_updates(package_name=""):
 
 
 def get_plugin(method, options):
-    """Return plugin object if CLI option is activated and method exists"""
+    """
+    Return plugin object if CLI option is activated and method exists
+
+    @param method: name of plugin's method we're calling
+    @type method: string
+
+    @param options: opt_parser options
+    @type options: list of opt_parser.parse_args
+
+    @returns: list of plugins with `method`
+
+    """
     all_plugins = []
     for entry_point in pkg_resources.iter_entry_points('yolk.plugins'):
         plugin_obj = entry_point.load()
@@ -113,7 +129,22 @@ def get_plugin(method, options):
     return all_plugins
 
 def show_distributions(show, project_name, version, options):
-    """Show list of installed activated OR non-activated packages"""
+    """
+    Show list of installed activated OR non-activated packages
+    @param show: type of pkgs to show (all, active or nonactive)
+    @type show: string
+
+    @param project_name: pkg_resources Distribution project name to query
+    @type project_name: string
+
+    @param version: pkg_resources Distribution version
+    @type version: string
+
+    @param options: opt_parser options
+    @type options: list of opt_parser.parse_args
+
+    @returns: None or 2 if error 
+    """
     show_metadata = options.metadata
     fields = options.fields
 
@@ -153,13 +184,16 @@ def show_distributions(show, project_name, version, options):
         else:
             print dist + " has no metadata"
         results = True
-    if not results:
+    if not results and project_name:
         if version:
             pkg_spec = "%s==%s" % (project_name, version)
         else:
             pkg_spec = "%s" % project_name
-
-        LOGGER.error("%s is not installed." % pkg_spec)
+        if show == "all":
+            LOGGER.error("There are no versions of %s installed." % pkg_spec)
+        else:
+            LOGGER.error("There are no %s versions of %s installed." % \
+                    (show, pkg_spec))
         return 2
     elif show == "all" and results and fields:
         print "Versions with '*' are non-active."
@@ -167,7 +201,27 @@ def show_distributions(show, project_name, version, options):
 
 
 def print_metadata(metadata, develop, active, options, installed_by):
-    """Print out formatted metadata"""
+    """
+    Print out formatted metadata
+    @param metadata: package's metadata
+    @type metadata:  pkg_resources Distribution obj
+
+    @param develop: path to pkg if its deployed in development mode
+    @type develop: string
+
+    @param active: show if package is activated or not
+    @type active: boolean
+
+    @param options: opt_parser options
+    @type options: list of opt_parser.parse_args
+
+    @param installed_by: Shows if pkg was installed by a package manager other
+                         than setuptools
+    @type installed_by: string
+
+    @returns: None
+    
+    """
     show_metadata = options.metadata
     fields = options.fields
 
@@ -221,14 +275,22 @@ def print_metadata(metadata, develop, active, options, installed_by):
 
 
 def show_deps(pkg_ver):
-    """Show dependencies for package(s)"""
+    """
+    Show dependencies for package(s)
+
+    @param pkg_ver: setuptools pkgspec (e.g. kid>=0.8)
+    @type pkg_ver: string
+    
+    @returns: None or 2 if error 
+    """
 
     if not pkg_ver:
         msg = \
             '''I need at least a package name.
 You can also specify a package name and version:
   yolk -d kid==0.8'''
-        raise Usage(msg)
+        LOGGER.error(msg)
+        return 2
 
     try:
         (project_name, ver) = pkg_ver[0].split('=')
@@ -266,26 +328,78 @@ You can also specify a package name and version:
 ##############################################################################
 
 
-def show_download_links(package_name, version, file_type):
-    """Query PyPI for pkg download URI for a packge"""
+def show_download_links(pkg_name, version, file_type):
+    """
+    Query PyPI for pkg download URI for a packge
 
+    @param pkg_name: pkg_resources Distribution project name to query
+    @type pkg_name: string
+
+    @param version: pkg_resources Distribution version
+    @type version: string
+
+    @param file_type: svn, source, binary, all (show all 3)
+    @param file_type: string
+
+    @returns: None
+    
+    """
+    source = True
+
+    if file_type == "svn":
+        version = "dev"
+        print_download_uri(pkg_name, version, source)
+    elif file_type == "source":
+        print_download_uri(pkg_name, version, source)
+    elif file_type == "binary":
+        source = False
+        print_download_uri(pkg_name, version, source)
+    elif file_type == "all":
+        #Search for source, binary and svn
+        source = True
+        print_download_uri(pkg_name, version, source)
+        source = False
+        print_download_uri(pkg_name, version, source)
+        source = True
+        print_download_uri(pkg_name, version, source)
+
+def print_download_uri(pkg_name, version, source):
+    """
+    @param pkg_name: pkg_resources Distribution project name to query
+    @type pkg_name: string
+
+    @param version: pkg_resources Distribution version
+    @type version: string
+
+    @param source: download source or binary
+    @type source: boolean
+
+    @returns: None
+
+    """
     url = None
     #Use setuptools monkey-patch to grab url
-    for url in get_download_uri(file_type, package_name, version):
-        print "%s" % url
-    #This should never happen because setuptools checks Cheese Shop?
-    if not url:
-        pypi = CheeseShop()
-        for url in pypi.get_download_urls(package_name, version, file_type):
-            print url
+    for url in get_download_uri(pkg_name, version, source):
+        if url:
+            print "%s" % url
 
-def browse_website(package_name, browser=None):
-    """Launch web browser at project's homepage"""
+def browse_website(pkg_name, browser=None):
+    """
+    Launch web browser at project's homepage
+    
+    @param pkg_name: pkg_resources Distribution project name
+    @type pkg_name: string
+
+    @param browser: name of web browser to use if not system default    
+    @type browser: string
+
+    @returns None
+    """
 
     pypi = CheeseShop()
     #Get verified name from pypi.
 
-    (pypi_project_name, versions) = pypi.query_versions_pypi(package_name)
+    (pypi_project_name, versions) = pypi.query_versions_pypi(pkg_name)
     if len(versions):
         metadata = pypi.release_data(pypi_project_name, versions[0])
         if metadata.has_key("home_page"):
@@ -294,57 +408,101 @@ def browse_website(package_name, browser=None):
                 browser = webbrowser.Konqueror()
             else:
                 browser = webbrowser.get()
-            try:
-                browser.open(metadata["home_page"], 2)
-            except AttributeError:
                 browser.open(metadata["home_page"], 2)
             return
 
     print "No homepage URL found."
 
 
-def show_pkg_metadata_pypi(package_name, version, fields):
-    """Show pkg metadata queried from PyPI"""
+def show_pkg_metadata_pypi(pkg_name, version, fields):
+    """
+    Show pkg metadata queried from PyPI
+
+    @param pkg_name: pkg_resources Distribution project name to query
+    @type pkg_name: string
+
+    @param version: pkg_resources Distribution version
+    @type version: string
+
+    @param fields: particular fields to show or "" for all
+    @type fields: string
+    
+    @returns: None or 2 if error 
+    
+    """
     pypi = CheeseShop()
     (pypi_project_name, versions) = \
-            pypi.query_versions_pypi(package_name, False)
+            pypi.query_versions_pypi(pkg_name, False)
+    if version and version in versions:
+        metadata = pypi.release_data(pypi_project_name, version)
+    else:
+        #Give highest version
+        metadata = pypi.release_data(pypi_project_name, versions[0])
 
-    if versions:
-        for ver in versions:
-            metadata = pypi.release_data(pypi_project_name, ver)
-            if metadata:
-                for key in metadata.keys():
-                    if not fields or (fields and fields==key):
-                        print "%s: %s" % (key, metadata[key])
-        
+    if metadata:
+        for key in metadata.keys():
+            if not fields or (fields and fields==key):
+                print "%s: %s" % (key, metadata[key])
     else:
         LOGGER.error(\
                 "I'm afraid we have no %s at The Cheese Shop. \
-                \nPerhaps a little red Leicester?" % package_name)
+                \nPerhaps a little red Leicester?" % pkg_name)
         return 2
 
 
-def get_all_versions_pypi(package_name, use_cached_pkglist=False):
-    """Fetch list of available versions for a package from The Cheese Shop"""
+def get_all_versions_pypi(pkg_name, my_version, use_cached_pkglist=False):
+    """
+    Fetch list of available versions for a package from The Cheese Shop
+
+    @param pkg_name: pkg_resources Distribution project name to query
+    @type pkg_name: string
+
+    @param my_version: pkg_resources Distribution version
+    @type my_version: string
+
+    @param use_cached_pkglist: use a pkg list stored on disk to avoid network
+                               usage
+    @type use_cached_pkglist: boolean
+
+    @returns: None or 2 if false
+    """
     pypi = CheeseShop()
     (pypi_project_name, versions) = \
-            pypi.query_versions_pypi(package_name, use_cached_pkglist)
+            pypi.query_versions_pypi(pkg_name, use_cached_pkglist)
 
-    #pypi_project_name may != package_name
+    #pypi_project_name may != pkg_name
     #it returns the name with correct case
     #i.e. You give beautifulsoup but PyPI knows it as BeautifulSoup
+    if my_version:
+        spec = "%s==%s" % (pkg_name, my_version)
+    else:
+        spec = pkg_name
 
-    if versions:
+    if versions and my_version in versions:
+        print_pkg_versions(pypi_project_name, [my_version])
+    elif not my_version:
         print_pkg_versions(pypi_project_name, versions)
     else:
         LOGGER.error(\
                 "I'm afraid we have no %s at The Cheese Shop. \
-                \nPerhaps a little red Leicester?" % package_name)
+                \nPerhaps a little red Leicester?" % spec)
         return 2
 
 
 def parse_search_spec(spec):
-    """Parse search args and return spec dict for PyPI"""
+    """
+    Parse search args and return spec dict for PyPI
+
+
+    @param spec: Cheese Shop package search spec
+                 e.g.
+                 name=Cheetah
+                 license=ZPL
+                 license=ZPL AND name=Cheetah
+    @type spec: string
+    
+    @returns:  
+    """
 
     usage = \
         """You can search PyPI by the following:
@@ -367,7 +525,8 @@ def parse_search_spec(spec):
       """
 
     if not spec:
-        raise Usage(usage)
+        LOGGER.error(usage)
+        return (None, None)
 
     try:
         spec = (" ").join(spec)
@@ -376,7 +535,6 @@ def parse_search_spec(spec):
         if " AND " in spec:
             (first, second) = spec.split('AND')
         elif " OR " in spec:
-
             (first, second) = spec.split('OR')
             operator = 'OR'
         else:
@@ -392,18 +550,33 @@ def parse_search_spec(spec):
         if second:
             spec[key2] = term2
     except:
-        raise Usage(usage)
+        LOGGER.error(usage)
+        spec = operator = None
     return (spec, operator)
 
 
-def pypi_search(arg, spec):
-    """Search PyPI by metadata keyword
-    e.g. yolk -S name=yolk
+def pypi_search(spec):
+    """
+    Search PyPI by metadata keyword
+    e.g. yolk -S name=yolk AND license=GPL
+
+    @param spec: Cheese Shop search spec
+    @type spec: list of strings
+    
+    spec could like like any of these:
+      ["name=yolk"]
+      ["license=GPL"]
+      ["name=yolk", "AND", "license=GPL"]
+
+    @returns: None
+
+
     """
     pypi = CheeseShop()
 
-    spec.insert(0, arg.strip())
     (spec, operator) = parse_search_spec(spec)
+    if not spec:
+        return 2
     for pkg in pypi.search(spec, operator):
         if pkg['summary']:
             summary = pkg['summary'].encode('utf-8')
@@ -416,7 +589,12 @@ def pypi_search(arg, spec):
 
 
 def get_rss_feed():
-    """Show last 20 package updates from PyPI RSS feed"""
+    """
+    Show last 20 package updates from PyPI RSS feed
+    
+    @returns: None
+    
+    """
 
     pypi = CheeseShop()
     rss = pypi.get_rss()
@@ -434,7 +612,18 @@ def get_rss_feed():
 
 
 def parse_pkg_ver(package_spec, installed):
-    """Return tuple with package_name and version from CLI args"""
+    """
+    Return tuple with package_name and version from CLI args
+
+    @param package_spec:
+    @type package_spec:
+    
+    @param installed: whether package is installed or not
+    @type installed: boolean
+
+    @returns: tuple(pkg_name, version) 
+    
+    """
 
     arg_str = ("").join(package_spec)
     if "==" not in arg_str:
@@ -453,28 +642,56 @@ def parse_pkg_ver(package_spec, installed):
         package_name = dists.case_sensitive_name(package_name)
     return (package_name, version)
 
-def print_pkg_versions(package_name, versions):
-    """Print list of versions available for a package"""
+def print_pkg_versions(pkg_name, versions):
+    """
+    Print list of versions available for a package
+
+    @param pkg_name: pkg_resources Distribution project name to query
+    @type pkg_name: string
+
+    @param version: pkg_resources Distribution version
+    @type version: string
+
+    @returns: None
+
+    """
 
     for ver in versions:
-        print "%s %s" % (package_name, ver)
+        print "%s %s" % (pkg_name, ver)
 
 def validate_pypi_opts(opt_parser):
-    """Check for sane parse options"""
+    """
+    Check for sane pkg_spec parse options
+
+    @returns: True if sane, False if insane
+    
+    """
 
     (options, remaining_args) = opt_parser.parse_args()
     if options.versions_available or options.query_metadata_pypi or \
         options.download_links or options.browse_website:
         if not remaining_args:
-            raise Usage, \
+            usage = \
                 """You must specify a package spec
 Examples:
   PackageName
   PackageName==2.0"""
+            LOGGER.error(usage)
+            return False
+        else:
+            return True
+    return True
 
 
 def show_entry_map(dist):
-    """Show entry map for a distribution"""
+    """
+    Show entry map for a distribution
+    
+    @param dist: `Distribution`
+    @param type: pkg_resources Distribution object
+    
+    @returns: None or 2 if error
+    """
     pprinter = pprint.PrettyPrinter()
     try:
         pprinter.pprint(pkg_resources.get_entry_map(dist))
@@ -483,7 +700,15 @@ def show_entry_map(dist):
         return 2
 
 def show_entry_points(module):
-    """Show entry points for a module"""
+    """
+    Show entry points for a module
+    
+    @param module: module name
+    @type type: string
+
+    @returns: None or 2 if error
+    
+    """
     found = False
     for entry_point in pkg_resources.iter_entry_points(module):
         found = True
@@ -501,7 +726,12 @@ def show_entry_points(module):
         return 2
 
 def setup_opt_parser():
-    """Setup the optparser"""
+    """
+    Setup the optparser
+
+    @returns: opt_parser.OptionParser
+    
+    """
 
     usage = "usage: %prog [options]"
     opt_parser = optparse.OptionParser(usage=usage)
@@ -586,8 +816,8 @@ def setup_opt_parser():
                           metavar='SEARCH_SPEC <AND/OR SEARCH_SPEC>')
 
     group_pypi.add_option("-T", "--file-type", action="store", dest=
-                          "file_type", default="all", help=
-                          "You may specify 'source', 'binary' or 'all' when using -D.")
+                          "file_type", default="source", help=
+                          "You may specify 'source', 'binary', 'svn' or 'all' when using -D.")
 
     group_pypi.add_option("-U", "--show-updates", action='store_true',
                           dest="show_updates", default=False, help=
@@ -613,12 +843,17 @@ def setup_opt_parser():
 
 
 def main():
-    """Parse options and perform actions"""
+    """
+    Parse options and perform actions
+    
+    @returns: 2 if error for sys.exit
+    """
 
     opt_parser = setup_opt_parser()
     (options, remaining_args) = opt_parser.parse_args()
 
-    validate_pypi_opts(opt_parser)
+    if not validate_pypi_opts(opt_parser):
+        return 2
     if not options.search and (len(sys.argv) == 1 or len(remaining_args) >
                                2):
         opt_parser.print_help()
@@ -646,7 +881,11 @@ def main():
         package = version = None
 
     if options.search:
-        return pypi_search(options.search, remaining_args)
+        #Add remainging cli arguments to options.search
+        spec = remaining_args
+        search_arg = options.search
+        spec.insert(0, search_arg.strip())
+        return pypi_search(spec)
     elif options.version:
         print "Version %s" % __version__.VERSION
         return
@@ -665,7 +904,7 @@ def main():
             opt_parser.error("Choose either -l, -n or -a")
         return show_distributions("nonactive", package, version, options)
     elif options.versions_available:
-        return get_all_versions_pypi(package, False)
+        return get_all_versions_pypi(package, version, False)
     elif options.browse_website:
         return browse_website(package)
     elif options.download_links:
