@@ -33,15 +33,14 @@ XMLRPC = True
 if sys.version_info[0] == 2 and sys.version_info[1] == 5:
     from xml.etree.ElementTree import parse
 else:
-    try:
-        from cElementTree import parse
-    except ImportError:
-        try:
-            from elementtree.ElementTree import parse
-        except ImportError:
-            #If there is absolutely no version of elementtree available
-            #this disables reading RSS feed (-L option).
-            XMLRPC = False
+    #For Python <=2.4
+    try: 
+        from elementtree.ElementTree import parse 
+    except ImportError: 
+        #If there is absolutely no version of elementtree available 
+        #this disables reading RSS feed (-L option). 
+        XMLRPC = False
+
 
 PYPI_SERVER = xmlrpclib.Server('http://cheeseshop.python.org/pypi')
 PYPI_URL = 'http://www.python.org/pypi?:action=rss'
@@ -60,7 +59,7 @@ class CheeseShop:
     def get_rss(self):
         """Fetch last 20 package release items from PyPI RSS feed"""
         if not XMLRPC:
-            print >> sys.stderr, "You need to install ElementTree to use -L"
+            print >> sys.stderr, "You need to install cElementTree to use -L"
             sys.exit(2)
         rss = {}
         request = urllib2.Request(PYPI_URL)
@@ -201,6 +200,10 @@ def filter_url(pkg_type, url):
     if "#md5=" in url:
         url = url.split("#")[0]
 
+    #pkg_spec==dev (svn)
+    if url.endswith("-dev"):
+        url = url.split("#egg=")[0]
+
     if pkg_type == "all":
         return url
 
